@@ -81,6 +81,8 @@ static void wnck_selector_dispose           (GObject           *object);
 static void wnck_selector_finalize          (GObject           *object);
 static void wnck_selector_realize           (GtkWidget *widget);
 static void wnck_selector_unrealize         (GtkWidget *widget);
+static gboolean wnck_selector_scroll_event  (GtkWidget        *widget,
+                                             GdkEventScroll   *event);
 static void wnck_selector_connect_to_window (WnckSelector      *selector,
                                              WnckWindow        *window);
 
@@ -156,7 +158,7 @@ wnck_selector_dimm_icon (GdkPixbuf *pixbuf)
   w = gdk_pixbuf_get_width (pixbuf);
   h = gdk_pixbuf_get_height (pixbuf);
 
-  if (gdk_pixbuf_get_has_alpha (pixbuf)) 
+  if (gdk_pixbuf_get_has_alpha (pixbuf))
     dimmed = gdk_pixbuf_copy (pixbuf);
   else
     dimmed = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
@@ -220,7 +222,7 @@ wnck_selector_set_window_icon (WnckSelector *selector,
     }
 
   if (window && wnck_window_is_minimized (window))
-    {      
+    {
       pixbuf = wnck_selector_dimm_icon (pixbuf);
       freeme2 = pixbuf;
     }
@@ -424,8 +426,8 @@ wnck_selector_window_state_changed (WnckWindow *window,
       gtk_menu_reposition (GTK_MENU (selector->priv->menu));
     }
 
-  if (changed_mask & 
-      (WNCK_WINDOW_STATE_DEMANDS_ATTENTION | WNCK_WINDOW_STATE_URGENT))      
+  if (changed_mask &
+      (WNCK_WINDOW_STATE_DEMANDS_ATTENTION | WNCK_WINDOW_STATE_URGENT))
     {
       if (wnck_window_or_transient_needs_attention (window))
 	_make_gtk_label_bold (GTK_LABEL (item->label));
@@ -509,7 +511,8 @@ wnck_selector_activate_window (WnckWindow *window)
 static gint
 wnck_selector_get_width (GtkWidget *widget, const char *text)
 {
-  GtkStyle *style;
+  GtkStyleContext *style_context;
+  GtkStateFlags state;
   PangoContext *context;
   PangoFontMetrics *metrics;
   gint char_width;
@@ -519,11 +522,12 @@ wnck_selector_get_width (GtkWidget *widget, const char *text)
   gint screen_width;
   gint width;
 
-  gtk_widget_ensure_style (widget);
-  style = gtk_widget_get_style (widget);
+  state = gtk_widget_get_state_flags (widget);
+  style_context = gtk_widget_get_style_context (widget);
 
   context = gtk_widget_get_pango_context (widget);
-  metrics = pango_context_get_metrics (context, style->font_desc,
+  metrics = pango_context_get_metrics (context,
+                                       gtk_style_context_get_font (style_context, state),
                                        pango_context_get_language (context));
   char_width = pango_font_metrics_get_approximate_char_width (metrics);
   pango_font_metrics_unref (metrics);
@@ -541,7 +545,7 @@ wnck_selector_get_width (GtkWidget *widget, const char *text)
   return width;
 }
 
-static void  
+static void
 wnck_selector_drag_begin (GtkWidget          *widget,
 			  GdkDragContext     *context,
 			  WnckWindow         *window)
@@ -561,7 +565,7 @@ wnck_selector_drag_begin (GtkWidget          *widget,
     _wnck_window_set_as_drag_icon (window, context, widget);
 }
 
-static void  
+static void
 wnck_selector_drag_data_get (GtkWidget          *widget,
 			     GdkDragContext     *context,
 			     GtkSelectionData   *selection_data,
@@ -569,7 +573,7 @@ wnck_selector_drag_data_get (GtkWidget          *widget,
 			     guint               time,
 			     WnckWindow         *window)
 {
-  gulong xid;    
+  gulong xid;
 
   xid = wnck_window_get_xid (window);
   gtk_selection_data_set (selection_data,
@@ -626,12 +630,12 @@ wnck_selector_item_new (WnckSelector *selector,
       g_signal_connect_object (item, "drag_data_get",
                                G_CALLBACK (wnck_selector_drag_data_get),
                                G_OBJECT (window),
-                               0); 
+                               0);
 
       g_signal_connect_object (item, "drag_begin",
                                G_CALLBACK (wnck_selector_drag_begin),
                                G_OBJECT (window),
-                               0); 
+                               0);
     }
 
   return item;
@@ -641,17 +645,19 @@ static void
 wnck_selector_workspace_name_changed (WnckWorkspace *workspace,
                                       GtkLabel      *label)
 {
-  GtkStyle *style;
-  GdkColor *color;
-  char     *name;
-  char     *markup;
+  GtkStyleContext *context;
+  GdkRGBA          color;
+  char            *name;
+  char            *markup;
 
-  style = gtk_widget_get_style (GTK_WIDGET (label));
-  color = &style->fg[GTK_STATE_INSENSITIVE];
+  context = gtk_widget_get_style_context (GTK_WIDGET (label));
+  gtk_style_context_get_color (context, GTK_STATE_FLAG_INSENSITIVE, &color);
 
   name = g_markup_escape_text (wnck_workspace_get_name (workspace), -1);
   markup = g_strdup_printf ("<span size=\"x-small\" style=\"italic\" foreground=\"#%.2x%.2x%.2x\">%s</span>",
-                            color->red, color->green, color->blue, name);
+                            (int)(color.red * 65535 + 0.5),
+                            (int)(color.green * 65535 + 0.5),
+                            (int)(color.blue * 65535 + 0.5), name);
   g_free (name);
 
   gtk_label_set_markup (label, markup);
@@ -659,9 +665,8 @@ wnck_selector_workspace_name_changed (WnckWorkspace *workspace,
 }
 
 static void
-wnck_selector_workspace_label_style_set (GtkLabel      *label,
-                                         GtkStyle      *previous_style,
-                                         WnckWorkspace *workspace)
+wnck_selector_workspace_label_style_updated (GtkLabel      *label,
+                                             WnckWorkspace *workspace)
 {
   wnck_selector_workspace_name_changed (workspace, label);
 }
@@ -686,8 +691,8 @@ wnck_selector_add_workspace (WnckSelector *selector,
   gtk_widget_show (label);
   /* the handler will also take care of setting the name for the first time,
    * and we'll be able to adapt to theme changes */
-  g_signal_connect (G_OBJECT (label), "style-set",
-                    G_CALLBACK (wnck_selector_workspace_label_style_set),
+  g_signal_connect (G_OBJECT (label), "style-updated",
+                    G_CALLBACK (wnck_selector_workspace_label_style_updated),
 		    workspace);
   wncklet_connect_while_alive (workspace, "name_changed",
                                G_CALLBACK (wnck_selector_workspace_name_changed),
@@ -704,7 +709,6 @@ wnck_selector_add_workspace (WnckSelector *selector,
 static GtkWidget *
 wnck_selector_create_window (WnckSelector *selector, WnckWindow *window)
 {
-  WnckWorkspace *workspace;
   GtkWidget *item;
   GtkWidget *image;
   char *name;
@@ -723,9 +727,6 @@ wnck_selector_create_window (WnckSelector *selector, WnckWindow *window)
                                  GTK_WIDGET (image));
   gtk_widget_show (image);
 
-  workspace =
-    wnck_screen_get_active_workspace (wnck_selector_get_screen (selector));
-
   g_signal_connect_swapped (item, "activate",
                             G_CALLBACK (wnck_selector_activate_window),
                             window);
@@ -775,7 +776,7 @@ wnck_selector_insert_window (WnckSelector *selector, WnckWindow *window)
       gtk_menu_shell_insert (GTK_MENU_SHELL (selector->priv->menu),
                              item, i);
     }
-  else 
+  else
     {
       workspace_n = wnck_workspace_get_number (workspace);
 
@@ -1019,10 +1020,10 @@ wnck_selector_destroy_menu (GtkWidget *widget, WnckSelector *selector)
 }
 
 static gboolean
-wnck_selector_scroll_cb (WnckSelector *selector,
-                         GdkEventScroll *event,
-                         gpointer user_data)
+wnck_selector_scroll_event (GtkWidget      *widget,
+                            GdkEventScroll *event)
 {
+  WnckSelector *selector;
   WnckScreen *screen;
   WnckWorkspace *workspace;
   GList *windows_list;
@@ -1031,6 +1032,8 @@ wnck_selector_scroll_cb (WnckSelector *selector,
   WnckWindow *previous_window;
   gboolean should_activate_next_window;
 
+  selector = WNCK_SELECTOR (widget);
+
   screen = wnck_selector_get_screen (selector);
   workspace = wnck_screen_get_active_workspace (screen);
   windows_list = wnck_screen_get_windows (screen);
@@ -1089,7 +1092,7 @@ wnck_selector_scroll_cb (WnckSelector *selector,
 
       previous_window = window;
     }
-  
+
   return TRUE;
 }
 
@@ -1186,10 +1189,8 @@ wnck_selector_on_show (GtkWidget *widget, WnckSelector *selector)
 static void
 wnck_selector_fill (WnckSelector *selector)
 {
-  GtkWidget *menu_item;
-
-  g_signal_connect (selector, "scroll-event",
-                    G_CALLBACK (wnck_selector_scroll_cb), selector);
+  GtkWidget      *menu_item;
+  GtkCssProvider *provider;
 
   menu_item = gtk_menu_item_new ();
   gtk_widget_show (menu_item);
@@ -1210,13 +1211,18 @@ wnck_selector_fill (WnckSelector *selector)
                     G_CALLBACK (wnck_selector_on_show), selector);
 
   gtk_widget_set_name (GTK_WIDGET (selector),
-                       "gnome-panel-window-menu-menu-bar-style");
+                       "gnome-panel-window-menu-menu-bar");
 
-  gtk_rc_parse_string ("style \"gnome-panel-window-menu-menu-bar-style\" {\n"
-                       "        GtkMenuBar::shadow-type = none\n"
-                       "        GtkMenuBar::internal-padding = 0\n"
-                       "}\n"
-                       "widget \"*gnome-panel-window-menu-menu-bar*\" style : highest \"gnome-panel-window-menu-menu-bar-style\"");
+  provider = gtk_css_provider_new ();
+  gtk_css_provider_load_from_data (provider,
+                                   "#gnome-panel-window-menu-menu-bar {\n"
+                                   " border-width: 0px;\n"
+                                   "}",
+                                   -1, NULL);
+  gtk_style_context_add_provider (gtk_widget_get_style_context (GTK_WIDGET (selector)),
+                                  GTK_STYLE_PROVIDER (provider),
+                                  GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+  g_object_unref (provider);
 
   gtk_widget_show (GTK_WIDGET (selector));
 }
@@ -1256,6 +1262,7 @@ wnck_selector_class_init (WnckSelectorClass *klass)
 
   widget_class->realize   = wnck_selector_realize;
   widget_class->unrealize = wnck_selector_unrealize;
+  widget_class->scroll_event = wnck_selector_scroll_event;
 }
 
 static GObject *
